/*
  Java gfx-test framework

   Copyright (C) 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.BasicStroke;
import java.awt.Graphics2D;



import org.gfxtest.callbacks.PolygonDrawCallbacks;
import org.gfxtest.framework.ColorPalette;
import org.gfxtest.framework.GrayscalePalette;
import org.gfxtest.framework.HSBPalette;
import org.gfxtest.framework.PrintTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test suite checks rendering of various polygons.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.PRINT_TEST)
@GraphicsPrimitive(GraphicsPrimitives.POLYGON)
@RenderStyle(RenderStyles.NORMAL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class PrintTestPolygons extends PrintTest
{
    /**
     * Method which renders set of polygons using various colors and stroke
     * styles. For each polygon, the callback function/method is called to
     * perform all required setup.
     * 
     * @param image
     *            image to which polygons are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @param step
     *            between two near polygons
     * @param polygonDrawCallbacks
     *            class containing set of callback methods
     */
    private static void drawPolygons(TestImage image, Graphics2D graphics, int step, PolygonDrawCallbacks polygonDrawCallbacks)
    {
        // setup rendering
        polygonDrawCallbacks.setup(image, graphics);

        // image width and height
        final int width = image.getWidth();
        final int height = image.getHeight();

        // horizontal coordinates of polygon endpoints
        int x1 = BORDER;
        int x2 = width - BORDER;

        // vertical coordinates of polygon endpoints
        int y1 = BORDER;
        int y2 = height - BORDER;

        // index to color palette
        int colorIndex = 0;

        // draw all polygons onto a paper
        while (x1 < x2 - BORDER && y1 < y2 - BORDER)
        {
            // setup can be made for each polygon
            polygonDrawCallbacks.iterationCallBack(x1, y1, x2, y2, colorIndex++);
            // render the polygon
            int xpoints[] = {x1, x2, x2, x1};
            int ypoints[] = {y1, y1, y2, y2};
            graphics.drawPolygon(xpoints, ypoints, 4);
            // setup coordinates for the next polygon
            x1 += step;
            y1 += step;
            x2 -= step;
            y2 -= step;
        }

        // cleanup rendering
        polygonDrawCallbacks.cleanup();
    }

    /**
     * Test basic behavior of method Graphics.drawPolygon(). Polygons are
     * rendered with default width and default end caps. Color of all rendered
     * polygons are set to black.
     * 
     * @param image
     *            image to which polygons are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawPolygonsBasicStyle(TestImage image, Graphics2D graphics2d)
    {
        drawPolygons(image, graphics2d, POLYGON_STEP, new PolygonDrawCallbacks()
        {
            /**
             * Callback function called before each polygon is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int index)
            {
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawPolygon(). Polygon are
     * rendered with default width and default end caps. Color of all rendered
     * polygons are selected from a palette.
     * 
     * @param image
     *            image to which polygons are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawPolygonsColorPalette(TestImage image, Graphics2D graphics2d)
    {
        drawPolygons(image, graphics2d, POLYGON_STEP, new PolygonDrawCallbacks()
        {
            /**
             * Callback function called before each polygon is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int index)
            {
                // set polygon color
                this.graphics.setColor(ColorPalette.getColor(index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawPolygon(). Polygon are
     * rendered with default width and default end caps. Color of all rendered
     * polygons are selected from a palette.
     * 
     * @param image
     *            image to which polygons are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawPolygonsColorPaletteInv(TestImage image, Graphics2D graphics2d)
    {
        drawPolygons(image, graphics2d, POLYGON_STEP, new PolygonDrawCallbacks()
        {
            /**
             * Callback function called before each polygon is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int index)
            {
                // set polygon color
                this.graphics.setColor(ColorPalette.getColor(MAX_COLOR_INDEX - index));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawPolygon(). Polygon are
     * rendered with default width and default end caps. Color of all rendered
     * polygons are selected from a grayscale palette.
     * 
     * @param image
     *            image to which polygons are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawPolygonsGrayscale(TestImage image, Graphics2D graphics2d)
    {
        drawPolygons(image, graphics2d, POLYGON_STEP, new PolygonDrawCallbacks()
        {
            /**
             * Callback function called before each polygon is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int index)
            {
                // compute grayscale value
                float gray = (y1 - BORDER) * 4.0f / this.image.getHeight();
                // set polygon color
                this.graphics.setColor(GrayscalePalette.createGrayscaleColor(gray));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawPolygon(). Polygon are
     * rendered with default width and default end caps. Color of all rendered
     * polygons are selected from a grayscale palette.
     * 
     * @param image
     *            image to which polygons are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawPolygonsGrayscaleInv(TestImage image, Graphics2D graphics2d)
    {
        drawPolygons(image, graphics2d, POLYGON_STEP, new PolygonDrawCallbacks()
        {
            /**
             * Callback function called before each polygon is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int index)
            {
                // compute grayscale value
                float gray = 1.0f - (y1 - BORDER) * 4.0f / this.image.getHeight();
                // set polygon color
                this.graphics.setColor(GrayscalePalette.createGrayscaleColor(gray));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawPolygon(). Polygon are
     * rendered with default width and default end caps. Color of all rendered
     * polygons are selected from a grayscale palette.
     * 
     * @param image
     *            image to which polygons are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawPolygonsColorScale(TestImage image, Graphics2D graphics2d)
    {
        drawPolygons(image, graphics2d, POLYGON_STEP, new PolygonDrawCallbacks()
        {
            /**
             * Callback function called before each polygon is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int index)
            {
                // compute hue value
                float hue = (y1 - BORDER) * 4.0f / this.image.getHeight();
                // set polygon color
                this.graphics.setColor(HSBPalette.createHsbColor(hue));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Test basic behavior of method Graphics.drawPolygon(). Polygon are
     * rendered with default width and default end caps. Color of all rendered
     * polygons are selected from a hue palette.
     * 
     * @param image
     *            image to which polygons are to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testDrawPolygonsColorScaleInv(TestImage image, Graphics2D graphics2d)
    {
        drawPolygons(image, graphics2d, POLYGON_STEP, new PolygonDrawCallbacks()
        {
            /**
             * Callback function called before each polygon is rendered.
             */
            @Override
            public void iterationCallBack(int x1, int y1, int x2, int y2, int index)
            {
                // compute hue value
                float hue = 1.0f - (y1 - BORDER) * 4.0f / this.image.getHeight();
                // set polygon color
                this.graphics.setColor(HSBPalette.createHsbColor(hue));
                return;
            }
        });

        // test return value
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     * 
     * @param args
     *            not used in this case
     */
    public static void main(String[] args)
    {
        new PrintTestPolygons().runTestSuite(args);
    }

}
